# -*- coding: utf-8 -*-
class BizException(Exception):
    def __init__(self, message: str):
        self.message = message
        self.code = -1


class NoConfigException(Exception):
    def __init__(self, message: str = '系统参数配置异常'):
        self.message = message
        self.code = -1



# -*- coding: utf-8 -*-
import json
import random
import uuid
from typing import List, Dict
from zcbot_url_parser import parser as url_parser
from ..util import logger, cfg
from ..util import time as time_lib
from ..util.exceptions import BizException
from ..client.redis_client import Redis
from ..util import redis_key as redis_key_util
from ..model.stream import StreamApiData, StreamTaskItem
from ..dao import batch_spider as spider_service

LOGGER = logger.get('流采')


def publish(api_data: StreamApiData):
    """
    准备批次采集任务物料
    """
    # 如果上游传递则直接使用，否则本系统生成批次编号
    app_code = api_data.appCode
    task_items = api_data.taskItems or []
    spider_ids = api_data.spiderIds
    task_queue_suffix = api_data.taskQueueSuffix
    if not app_code:
        raise BizException(f'[分拣]app_code未指定 api_data={api_data}')
    if not task_items:
        raise BizException(f'[分拣]任务清单为空 api_data={api_data}')
    spider_ids_string = ",".join(spider_ids)
    LOGGER.info(f'[分拣]开始流采分拣 app_code={app_code}, spider_ids={spider_ids_string}, rows={len(task_items)}')
    # 物料分拣
    result_map, classify_result_list, classify_ignore_list = classify(api_data)
    # 初始化任务队列
    init_job(result_map, spider_ids, task_queue_suffix)
    LOGGER.info(f'[分拣]流采分拣完成 app_code={app_code}, spider_ids={spider_ids_string}, rows={len(task_items)}')

    return result_map, classify_result_list, classify_ignore_list


# def simple_publish(api_data: StreamApiData):
#     """
#     准备批次采集任务物料
#     """
#     # 如果上游传递则直接使用，否则本系统生成批次编号
#     app_code = api_data.app_code
#     task_items = api_data.task_items or []
#     task_type = api_data.task_type
#
#     if not app_code:
#         raise BizException(f'[分拣]app_code未指定 api_data={api_data}')
#     if not task_items:
#         raise BizException(f'[分拣]任务清单为空 api_data={api_data}')
#
#     LOGGER.info(f'[分拣]开始流采分拣 app_code={app_code}, task_type={task_type}, rows={len(task_items)}')
#     # 简易分拣
#     result_map = dict()
#     for idx, row in enumerate(task_items):
#         if row.sn and row.platCode:
#             # 额外字段
#             row.appCode = row.appCode or app_code
#             task_list = result_map.get(row.platCode)
#             if not task_list:
#                 task_list = list()
#                 result_map[row.platCode] = task_list
#             task_list.append(row)
#
#     # 初始化任务队列
#     init_job(result_map, task_type)
#     LOGGER.info(f'[分拣]流采分拣完成 app_code={app_code}, task_type={task_type}, rows={len(task_items)}')
#
#     return result_map, task_items


def classify(api_data: StreamApiData) -> (Dict[str, List[StreamTaskItem]], List[dict]):
    """
    任务分拣
    :param api_data: 待分拣任务清单，
    :return:
    """
    app_code = api_data.appCode

    todo_list = api_data.taskItems

    LOGGER.info('[分拣]开始流采分拣 total=%s' % len(todo_list))

    result_map = dict()
    classify_result_list = list()
    classify_ignore_list = list()
    for idx, row in enumerate(todo_list):
        plat_code = row.platCode
        # 如果有plat_code则不分拣(兼容购物党等平台)
        if not plat_code:
            # 解析链接组装任务
            url_model = url_parser.parse_url(row.url)
            link_id, plat_code, plat_name, ec_sku_id = url_model.link_sn, url_model.plat_code, url_model.plat_name, url_model.ec_sku_id
            if link_id and plat_code and ec_sku_id:
                row.linkId = link_id
                row.platCode = plat_code
                row.platName = plat_name
                row.ecSkuId = ec_sku_id

            else:
                LOGGER.warning(f'[分拣]未识别的流采任务 url={row.url}, url_sn={link_id}, plat_code={plat_code}, ec_sku_id={ec_sku_id}')
                classify_ignore_list.append(row)
                continue
        # 额外字段
        row.appCode = row.appCode or app_code
        row.rowId = row.rowId or row.sn
        # 分拣
        if row.sn and plat_code:
            task_list = result_map.get(plat_code)
            if not task_list:
                task_list = list()
                result_map[plat_code] = task_list
            task_list.append(row)
            classify_result_list.append(row.to_classify_result())
    # 统计
    count_map = dict()
    for key in result_map.keys():
        count_map[key] = len(result_map.get(key))
    LOGGER.info('[分拣]流采分拣完成 %s' % count_map)

    return result_map, classify_result_list, classify_ignore_list


def init_job(result_map: Dict[str, List[StreamTaskItem]], spider_ids: List[str], task_queue_suffix: str) -> (Dict[str, List[StreamTaskItem]], List[dict], str):
    # 初始化任务队列
    for spider_id in spider_ids:
        for plat_code in result_map.keys():
            plat_task_list = result_map.get(plat_code)
            if task_queue_suffix:
                redis_key = f'zcbot:stream:{spider_id}'
            else:
                redis_key = f'zcbot:stream:{spider_id}:{task_queue_suffix}'

            plat_init_count = init_task_queue(redis_key, plat_task_list)
            LOGGER.info(f'[发布]初始化流采队列 plat_code={plat_code}, plat_init_count={plat_init_count}')


def init_task_queue(redis_key, spider_conf, rows) -> int:
    """
    初始化任务队列
    :param spider_conf:
    :param rows:
    :return:
    """
    # 初始化队列
    task_mode = spider_conf.get('task_mode', None)
    redis_key = spider_conf.get('redis_key', None)
    LOGGER.info(f'[队列]任务队列初始化开始 redis_key={redis_key}, task_mode={task_mode}')
    if task_mode and redis_key and task_mode == 'batch':
        # 批量模式（如：京东价格）
        return _init_redis_batch(redis_key, rows)
    else:
        # 单条采集模式
        return _init_redis(redis_key, rows)


def add_to_task_mapper(request_id, task_data, task_queue_key):
    """
    加入任务映射队列，用于重试任务源数据
    """
    rds = Redis()
    _data = {
        'queue': task_queue_key,
        'source': task_data,
        'genTime': time_lib.current_timestamp10()
    }
    _expire_seconds = cfg.get_int('ZCBOT_CORE_REDIS_QUEUE_EXPIRE') or 12 * 3600
    rds.client.set(redis_key_util.get_retry_request_source_key(request_id), json.dumps(_data), ex=_expire_seconds)


def _init_redis(redis_key, rows: List[StreamTaskItem]):
    rds = Redis()
    for row in rows:
        request_id = str(uuid.uuid4())
        task = {
            'requestId': request_id,
            "sn": str(row.sn),
            "url": row.url,
            "platCode": row.platCode,
            "rowId": str(row.rowId),
            "appCode": row.appCode,
            "callback": row.callback,
        }
        # 任务入队
        rds.client.lpush(redis_key, json.dumps(task))
        # 加入重试源数据集合
        add_to_task_mapper(request_id, task, redis_key)

    count = rds.client.llen(redis_key)
    LOGGER.info(f'[队列]任务队列初始化完成 -> 单条模式 key={redis_key}, row={count}, count={len(rows)}')

    return count


def _init_redis_batch(redis_key, rows: List[StreamTaskItem]):
    # TODO 待改
    BATCH_SIZE = 60
    _expire_seconds = cfg.get_int('ZCBOT_CORE_REDIS_QUEUE_EXPIRE') or 12 * 3600
    rds = Redis()
    batch_list = []
    random.shuffle(rows)
    for row in rows:
        task = {
            "sn": str(row.sn),
            "url": row.url,
            "platCode": row.platCode,
            "rowId": str(row.rowId),
            "appCode": row.appCode,
            "callback": row.callback,
        }
        batch_list.append(task)
        if len(batch_list) >= BATCH_SIZE:
            request_id = str(uuid.uuid4())
            batch_row_data = {
                'requestId': request_id,
                'data': batch_list
            }
            # 任务入队
            rds.client.lpush(redis_key, json.dumps(batch_row_data))
            # 加入重试源数据集合
            add_to_task_mapper(request_id, batch_row_data, redis_key)
            batch_list = []
    if batch_list:
        request_id = str(uuid.uuid4())
        batch_row_data = {
            'requestId': request_id,
            'data': batch_list
        }
        # 任务入队
        rds.client.lpush(redis_key, json.dumps(batch_row_data))
        # 加入重试源数据集合
        add_to_task_mapper(request_id, batch_row_data, redis_key)

    count = rds.client.llen(redis_key)
    LOGGER.info(f'[队列]任务队列初始化完成 -> 批量模式 key={redis_key}, row={count}, count={len(rows)}')

    return len(rows)



from typing import List, Union, Dict
from pydantic import BaseModel, Field
from .batch import BaseData
from .enums import TaskType


class StreamTaskItem(BaseData):
    """
    任务物料模型
    """
    # 【输入】对象唯一序列号（全局唯一，可用于主键，等于_id）
    sn: str = Field(description="对象唯一序列号", default=None)
    # 【输入】商品链接
    url: str = Field(description="商品链接", default=None)
    # 【输入】来源APP（流采模式）
    appCode: str = Field(description="应用编码", default=None)
    # 电商链接唯一码（platCode:ecSkuId）
    linkId: str = Field(description="电商链接唯一码", default=None)
    # 电商商品编码
    ecSkuId: str = Field(description="电商商品编码", default=None)
    # 电商平台编码（识别链接获得）
    platCode: str = Field(description="电商平台编码", default=None)
    # 电商平台名称（识别链接获得）
    platName: str = Field(description="电商平台名称", default=None)
    # 【输入】用户指定编号（可选，不填使用默认值，等于sn）
    rowId: str = Field(description="用户指定编号（可选，不填使用默认值，等于sn）", default=None)
    # 扩展字段，可是任意内容，透传
    callback: Union[str, Dict, List] = Field(description="扩展字段", default=None)

    def to_classify_result(self):
        rs = {
            'sn': self.sn,
            'linkId': self.linkId,
            'ecSkuId': self.ecSkuId,
            'platCode': self.platCode,
            'platName': self.platName,
            'appCode': self.appCode,
        }
        # 可有可无
        if self.rowId is not None:
            rs['rowId'] = self.rowId
        if self.callback is not None:
            rs['callback'] = self.callback
        return rs


class StreamApiData(BaseModel):
    """
    流式通用数据接收模型
    """
    # 【输入】应用编码
    appCode: str = Field(description="应用编码", default=None)
    # 【输入】任务类型
    spiderIds: List[str] = Field(description="爬虫主键", default=[])
    # 【输入】任务明细清单
    taskItems: List[StreamTaskItem] = Field(description="任务明细清单", default=[])
    # 【输入】文件名称配置键
    fileNameConfig: str = Field(description="任务明细清单", default='default')
    # 【输入】任务队列后缀（用于拆分某些应用专用任务队列）
    taskQueueSuffix: str = Field(description="任务队列后缀", default=None)



# -*- coding: utf-8 -*-
from typing import List
from ..model.enums import CommonStatus
from ..util import logger
from ..dao import stream_spider as stream_spider_dao
from ..model.entity import StreamSpider

LOGGER = logger.get('流采爬虫')


def get_spiders(spider_ids: List[str], status: CommonStatus = CommonStatus.ON) -> List[StreamSpider]:
    rows = stream_spider_dao.get_stream_spider(spider_ids=spider_ids, status=status)
    rows = [StreamSpider(**x) for x in rows]
    return rows



# -*- coding: utf-8 -*-
from typing import List
from ..client.mongo_client import Mongo
from ..model.base import Rule


# 获取链接分拣规则配置
def get_url_parse_rule(host: str = None):
    if host:
        return Rule(**Mongo().get(collection='zcbot_url_parse_rule', query={'_id': host}))

    return [Rule(**x) for x in Mongo().list(collection='zcbot_url_parse_rule')]


# 获取支持网站平台
def get_platforms():
    return Mongo().list(collection='zcbot_platforms', sort=[('sort', 1)])


# 获取文件命名规则配置
def get_file_name_config(config_id):
    return Mongo().get(collection='zcbot_file_name_config', query={'_id': config_id})



from typing import List, Union, Dict
from pydantic import BaseModel, Field
from .base import BaseData
from .enums import CrawlJobStatus
from ..util import time as time_lib


class BatchTaskItem(BaseData):
    """
    【输入】任务明细模型
    """
    # 【输入】对象唯一序列号（全局唯一，可用于主键，等于_id）
    sn: str = None
    # 【输入】用户指定编号
    rowId: str = None
    # 【输入】商品链接
    url: str = None
    # 扩展字段，可是任意内容，透传
    callback: Union[str, Dict, List] = None


class BatchTask(BaseModel):
    """
    【输入】批次采集数据接收模型
    """
    # 【输入】任务爬虫编码清单
    spiderId: Union[List[str], str] = None
    # 【输入】任务链接明细清单
    taskItems: List[BatchTaskItem] = None
    # 【输入】启动容器数量
    containerCount: int = 1
    # 【输入】文件名称配置键（主详图采集命名使用，非必要）
    fileNameConfig: str = 'default'


class BatchApiData(BaseModel):
    """
    【输入】批次采集接口接收模型
    """
    # 【输入】批次编号
    batchId: str
    # 【输入】应用编码
    appCode: str
    # 【输入】任务明细清单
    taskList: List[BatchTask]
    # 【输入】是否为补采任务
    supplyTask: bool = False


class BatchJobInst(BaseData):
    """
    【输出】任务明细模型
    """
    # 【输出】对象唯一序列号（全局唯一，可用于主键，等于_id）
    jobId: str = None
    # 【输出】批次编号
    batchId: str = None
    # 【输出】爬虫容器ID
    containerId: str = None
    # 【输出】爬虫实例指纹，用于爬虫内部标记结果出处
    instSn: str = None
    # 【输出】任务数
    itemCount: int = None
    # 【输出】job运行节点
    node: Dict = None
    # 【输出】平台编码
    platCode: str = None
    # 【输出】 爬虫Id
    spiderId: str = None
    # 【输出】job状态
    status: str = None
    # 【输出】job状态描述
    statusText: str = None
    # 创建时间
    genTime: int = Field(
        default_factory=time_lib.current_timestamp10
    )
    # 更新时间
    updateTime: int = Field(
        default_factory=time_lib.current_timestamp10
    )

    def set_status(self, status: CrawlJobStatus):
        self.status = status.name
        self.statusText = status.value


class BatchTaskResp(BaseModel):
    """
    【输出】批次采集数据接收模型
    """
    # 【输出】爬虫编码
    spiderId: str
    # 【输出】批次编号
    batchId: str
    # 【输出】任务链接明细清单
    nodes: List[BatchJobInst] = []


class BatchSpiderGroupQuery(BaseModel):
    """
    【输出】支持平台
    """
    # 【输入】平台编码
    platCodes: List[str] = None
    # 【输入】组编码
    groupCode: str = None
    # 【输入】是否可用
    # enable: int = 1
    status: str = None
    statusText: str = None



import time
from enum import Enum
from typing import Union, Dict, List
from pydantic import BaseModel


class MsgType(Enum):
    """
    消息类型枚举
    """
    # 爬虫控制指令
    ACT_OPENED = 'OPENED'
    ACT_CLOSED = 'CLOSED'
    # 数据消息
    DATA_SKU_TEXT = 'SKU_TEXT'
    DATA_SKU_IMAGES = 'SKU_IMAGES'
    # 压缩指令
    ZIP_DONE = 'ZIP_DONE'


class SignalType(Enum):
    """
    采云间爬虫状态信号枚举
    """
    OPENED = 'OPENED'
    CLOSED = 'CLOSED'


class BaseMsg(BaseModel):
    """
    标准消息数据模型
    """
    # 消息类型（控制消息：ACT；数据消息：DATA）
    msg_type: MsgType
    # 消息体
    msg_body: Union[str, Dict, List] = None
    # 产生时间戳
    gen_time: int = int(time.time())



# -*- coding: utf-8 -*-
from typing import List
from ..model.base import Platforms
from ..util import logger
from ..dao import batch_spider_group as batch_spider_group_dao
from ..model.entity import BatchSpiderGroup

LOGGER = logger.get('批次爬虫组')


def get_spider_group_list(group_code: str = None, plat_codes: List[str] = None, enable: int = None):
    rows = batch_spider_group_dao.get_spider_group_list(group_code=group_code, plat_codes=plat_codes, enable=enable)
    rows = [BatchSpiderGroup(**x) for x in rows]
    return rows


def get_spider_group(group_code: str = None, plat_code: str = None, enable: int = None):
    row = batch_spider_group_dao.get_spider_group(group_code=group_code, plat_code=plat_code, enable=enable)
    row = BatchSpiderGroup(**row)
    return row


def get_platforms_by_group(group_code: str, enable: int = None):
    rows = batch_spider_group_dao.get_platforms_by_group_code(group_code=group_code, enable=enable)
    rows = [Platforms(**x) for x in rows]
    return rows



from distutils.core import setup
from setuptools import find_packages

with open('README.rst', 'r') as f:
    long_description = f.read()

setup(name='zcbot-crawl-core',
      version='2.0.3',
      description='zcbot crawl core for zsodata',
      long_description=long_description,
      author='zsodata',
      author_email='team@zso.io',
      url='http://www.zsodata.com',
      install_requires=['zcbot-url-parser'],
      python_requires='>=3.7',
      license='BSD License',
      packages=find_packages(),
      platforms=['all'],
      include_package_data=True
      )



# -*- coding: utf-8 -*-
import json
import random
import uuid
from typing import List
from ..util import logger, cfg
from ..util import time as time_lib
from ..util import sequence as sequence_util
from ..util import redis_key as redis_key_util
from ..util.exceptions import BizException
from ..model.batch import BatchTaskItem
from ..client.redis_client import Redis
from ..client.portainer_client import PortainerClient
from ..model.entity import PortainerNode
from ..model.batch import BatchApiData, BatchJobInst
from ..model.enums import CommonStatus, CrawlJobStatus
from ..dao import batch_spider as spider_dao
from ..dao import node as node_dao

LOGGER = logger.get("任务")
client = PortainerClient()


def publish(api_data: BatchApiData):
    """
    发起批次采集
    """
    # 如果上游传递则直接使用，否则本系统生成批次编号
    batch_id = api_data.batchId or sequence_util.gen_batch_id()
    app_code = api_data.appCode
    LOGGER.info(f"[启动]开始启动 app_code={app_code}, batch_id={batch_id}")

    rs_list = list()

    # 任务数据与爬虫配置校验
    for task in api_data.taskList:
        task_items = task.taskItems
        spider_id = task.spiderId
        file_name_config = task.fileNameConfig
        if not spider_id or not task_items:
            LOGGER.error(f"[启动]存在异常任务 batch_id={batch_id}, spider_id={spider_id}, task_items={len(task_items)}, app_code={app_code}")
            continue
        spider = spider_dao.get_batch_spider(spider_id)
        if not spider or spider.status != CommonStatus.ON.name:
            LOGGER.error(f"[启动]爬虫不存在或已停用 spider_id={spider_id}, app_code={app_code}, batch_id={batch_id}, task_items={len(task_items)}")
            continue

        # 创建容器 + 初始化队列
        plat_code = spider.platCode
        task_mode = spider.taskMode
        batch_size = spider.batchSize
        inst_sn = sequence_util.short_uuid()
        # 初始化容器
        init_task_queue(task_items, batch_id, spider_id, batch_size, plat_code, task_mode)

        # 选择一个节点
        node = node_balance_choice(spider.nodes)
        if not node:
            LOGGER.error(f"[启动]爬虫暂无可用运行节点 spider_id={spider_id}, nodes={spider.nodes}, app_code={app_code}, batch_id={batch_id}, task_items={len(task_items)}")
            continue

        # 创建容器
        container_name = f'{spider.taskType}-{spider.platCode}-{batch_id}-{inst_sn}'
        container_id = client.create_container(
            spider=spider,
            node=node,
            container_name=container_name,
            param_data={"batch_id": batch_id, "inst_sn": inst_sn, "app_code": app_code, "file_name_config": file_name_config}
        )
        # 附带信息
        item_count = len(task_items)
        if container_id:
            job_inst = BatchJobInst(
                jobId=f'{node.endpointId}:{container_id}',
                batchId=batch_id,
                containerId=container_id,
                instSn=inst_sn,
                itemCount=item_count,
                node=node,
                spiderId=spider_id,
                platCode=plat_code,
                status=CrawlJobStatus.ERROR.name,
                statusText=CrawlJobStatus.ERROR.value,
            )
            # 启动容器
            rs = client.start(node, container_id)
            if rs:
                # 启动成功
                job_inst.status = CrawlJobStatus.RUNNING.name
                job_inst.statusText = CrawlJobStatus.RUNNING.value
            rs_list.append(job_inst)

    return rs_list


def cancel_job(node_id: str, container_id: str):
    """
    取消采集任务
    """
    node = node_dao.get_node(node_id)
    if not node:
        raise BizException(f"[任务]节点不存在或已停用 node_id={node_id}, container_id={container_id}")

    rs = client.stop(node, container_id)
    LOGGER.info(f"[任务]取消完成 endpoint_id={node.endpointId}, container_id={container_id}, rs={rs}")
    return rs


def init_task_queue(task_items, batch_id, spider_id, batch_size, plat_code, task_mode) -> int:
    """
    初始化任务队列
    :param task_items:
    :param batch_id:
    :param batch_size:
    :param spider_id:
    :param app_code:
    :param task_mode:
    :param expire_seconds:
    :return:
    """

    # 初始化队列
    LOGGER.info(f"[队列]任务队列初始化开始 task_mode={task_mode}")
    tmp_queue_expire = cfg.get_int("ZCBOT_CORE_REDIS_TMP_QUEUE_EXPIRE") or 3600
    if task_mode and task_mode == "multi":
        # 批量模式（如：京东价格）
        return _init_redis_batch(
            batch_id=batch_id,
            batch_size=batch_size,
            spider_id=spider_id,
            plat_code=plat_code,
            rows=task_items,
            expire_seconds=tmp_queue_expire
        )
    else:
        # 单条采集模式
        return _init_redis(batch_id=batch_id, spider_id=spider_id, plat_code=plat_code, rows=task_items, expire_seconds=tmp_queue_expire)


def _init_redis(batch_id: str, spider_id: str, plat_code: str, rows: List[BatchTaskItem], expire_seconds: int):
    redis_key = redis_key_util.get_task_queue_key(batch_id, spider_id)
    Redis().client.delete(redis_key)
    for row in rows:
        request_id = str(uuid.uuid4())
        task = row.dict()
        task['requestId'] = request_id
        if plat_code:
            task["platCode"] = plat_code
        # 任务入队
        Redis().client.lpush(redis_key, json.dumps(task))
        # 加入重试源数据集合
        add_to_retry_source_mapper(request_id, task, redis_key)

    # 设置任务队列过期时间：默认12小时自动清理
    set_expire(batch_id=batch_id, spider_id=spider_id, expire_seconds=expire_seconds)
    count = Redis().client.llen(redis_key)
    LOGGER.info(f"[队列]任务队列初始化完成 -> 单条模式 key={redis_key}, row={count}, count={len(rows)}")

    return count


def _init_redis_batch(batch_id: str, batch_size: int, spider_id: str, plat_code: str, rows: List[BatchTaskItem], expire_seconds: int):
    redis_key = redis_key_util.get_task_queue_key(batch_id, spider_id)
    Redis().client.delete(redis_key)

    batch_list = []
    random.shuffle(rows)
    for row in rows:
        task = row.dict()
        if plat_code:
            task["platCode"] = plat_code
        batch_list.append(task)
        if len(batch_list) >= batch_size:
            request_id = str(uuid.uuid4())
            batch_row_data = {
                "requestId": request_id,
                "data": batch_list
            }
            # 任务入队
            Redis().client.lpush(redis_key, json.dumps(batch_row_data))
            # 加入重试源数据集合
            add_to_retry_source_mapper(request_id, batch_row_data, redis_key)
            batch_list = []
    if batch_list:
        request_id = str(uuid.uuid4())
        batch_row_data = {
            "requestId": request_id,
            "data": batch_list
        }
        # 任务入队
        Redis().client.lpush(redis_key, json.dumps(batch_row_data))
        # 加入重试源数据集合
        add_to_retry_source_mapper(request_id, batch_row_data, redis_key)

    # 设置任务队列过期时间：默认12小时自动清理
    set_expire(batch_id, spider_id, expire_seconds)
    count = Redis().client.llen(redis_key)
    LOGGER.info(f"[队列]任务队列初始化完成 -> 批量模式 key={redis_key}, row={count}, count={len(rows)}")

    return len(rows)


def set_expire(batch_id: str, spider_id: str, expire_seconds=None):
    """
    设置任务队列过期时间
    1、分拣后入库：默认1小时（job未启动则清理）
    2、job启动：重置过期时间（限制采集时效）
    :param batch_id:
    :param spider_id:
    :param expire_seconds:
    :return:
    """
    _expire_seconds = expire_seconds or cfg.get_int("ZCBOT_CORE_REDIS_QUEUE_EXPIRE") or 12 * 3600
    redis_key = redis_key_util.get_task_queue_key(batch_id, spider_id)
    Redis().client.expire(redis_key, _expire_seconds)


def add_to_retry_source_mapper(request_id, task_data, task_queue_key):
    """
    加入任务映射队列，用于重试任务源数据
    """
    _data = {
        "queue": task_queue_key,
        "source": task_data,
        "genTime": time_lib.current_timestamp10()
    }
    _expire_seconds = cfg.get_int("ZCBOT_CORE_REDIS_QUEUE_EXPIRE") or 12 * 3600
    Redis().client.set(redis_key_util.get_retry_request_source_key(request_id), json.dumps(_data), ex=_expire_seconds)


# 节点选择，这里未来可做节点负载均衡
def node_balance_choice(nodes: List[str]) -> PortainerNode:
    node_list = node_dao.get_node_list({'portainerId': {'$in': nodes}})
    if node_list:
        node = random.choice(node_list)
    else:
        node = ""
    return node
